@@ -5,6 +5,7 @@ import threading
 import queue
 import asyncio
 import logging
+import time
 
 logger = logging.getLogger(__name__)
 
@@ -75,7 +76,8 @@ class ProcessAudioCapture:
 
         self._thread: Optional[threading.Thread] = None
         self._stop_event = threading.Event()
-        self._async_queue: "queue.Queue[bytes | None]" = queue.Queue()
+        # Bounded queue to prevent unbounded memory growth (~1 second of audio at 10ms chunks)
+        self._async_queue: "queue.Queue[bytes | None]" = queue.Queue(maxsize=100)
 
     # --- public API -----------------------------------------------------
 
@@ -222,7 +224,8 @@ class ProcessAudioCapture:
                 continue
 
             if not data:
-                # パケットがまだ無いケース。ここで sleep 入れるかは後で調整。
+                # No data available yet, sleep briefly to reduce CPU usage
+                time.sleep(0.001)  # 1ms sleep
                 continue
 
             # callback
